@@ -18,14 +18,21 @@
 */
 
 #include <string.h>
+#include <assert.h>
+
 #include "buffer.h"
 
+struct _Buffer {
+	char*  	  _data;
+	unsigned int    _len;
+};
+
 Buffer*
 buffer_new (void)
 {
 	Buffer *new_buffer;
 
-	new_buffer = (Buffer*) malloc (sizeof(Buffer));
+	new_buffer = (Buffer*)malloc (sizeof(Buffer));
 	if (!new_buffer)
 		return NULL;
 
@@ -40,42 +47,19 @@ void
 buffer_destroy (Buffer *buffer)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
 	
-	buffer->_len = -1;
-
+	buffer->_len = 0;
 	free (buffer->_data);
+	buffer->_data = NULL;
+
 	free (buffer);
 }
 
 
-
 int
-buffer_set_data (Buffer* buffer, const char* data, int len)
+buffer_manage_data (Buffer* buffer, char* data, unsigned int len)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0);
-	assert (data || len == 0);
-	
-	buffer->_data = (char*) realloc (buffer->_data, len);
-	if (!buffer->_data)
-		return -1;
-		
-	buffer->_data = memmove (buffer->_data, data, len);
-	buffer->_len  = len;
-
-	return len;
-}
-
-
-
-int
-buffer_manage_data (Buffer* buffer, char* data, int len)
-{
-	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0);
 	assert (data || len == 0);
 
 	free (buffer->_data);
@@ -92,9 +76,8 @@ char
 buffer_at (const Buffer* buffer, BufferIterator pos)
 {       
 	assert (buffer);
-	assert (buffer->_len > 0);
 	assert (buffer->_data);
-	assert (pos >= 0 && pos < buffer_end(buffer));
+	assert (pos >= buffer_begin(pos) && pos < buffer_end(buffer));
 	
 	return buffer->_data[pos];
 }
@@ -105,11 +88,27 @@ const char*
 buffer_data (const Buffer* buffer)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
 
 	return buffer->_data;
 }
-	
+
+char *
+buffer_data_to_string (const Buffer* buffer)
+{
+	char *data;
+
+	assert (buffer);
+
+	data = (char*)malloc(buffer->_len + 1);
+	if (!data)
+		return NULL;
+
+	memcpy (data, buffer->_data, buffer->_len);
+	data[buffer->_len] = '\0';
+
+	return data;
+}
+
 
 
 const char* 
@@ -117,85 +116,46 @@ buffer_data_pos (const Buffer* buffer, BufferIterator pos)
 {
 	assert (buffer);
 	assert (buffer->_data);
-	assert (buffer->_len >= 0);
+	assert (pos < buffer_length(buffer));
 	assert (pos >= 0 && pos < buffer_end(buffer));
-	
+
 	return buffer->_data + pos;
 }
 
-
-
-int 
+unsigned int 
 buffer_length (const Buffer *buffer)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
 
 	return buffer->_len;
 }
 
 	
 int
-buffer_append (Buffer* buffer, const char* data, int len)
+buffer_append (Buffer* buffer, const char* data, unsigned int len)
 {
+	char *newdata;
+
 	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0);
 	assert (data || len == 0);
-	
-	buffer->_data = (char*)realloc (buffer->_data, buffer->_len + len);
-	if (!buffer->_data)
+
+	newdata = (char*)realloc(buffer->_data, buffer->_len + len);
+	if (!newdata)
 		return -1;
-
-	memcpy (buffer->_data + buffer->_len, data, len);
-
-	buffer->_len += len;
-
-	return buffer->_len;
-}
-
-int 
-buffer_prepend (Buffer *buffer, const char *data, int len)
-{
-	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0);
-	assert (data || len == 0);
+		
+	memcpy (newdata + buffer->_len, data, len);
 	
-	return buffer_insert (buffer, data, len, 0);
-}
-
-
-int
-buffer_insert (Buffer *buffer, const char *data, int len, int pos)
-{
-	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0 && pos >= 0);
-	assert (data || len == 0);
-	assert (buffer->_len > pos);
-	
-	buffer->_data = (char*)realloc (buffer->_data, buffer->_len + len);
-	if (!buffer->_data)
-		return -1;
-
-	memmove (buffer->_data + pos + len, 
-		buffer->_data + pos, 
-		buffer->_len - pos);
-	memmove (buffer->_data + pos, data, len);
-
+	buffer->_data = newdata;
 	buffer->_len += len;
 
 	return buffer->_len;
 }
 
 
-int
-buffer_erase (Buffer *buffer, int pos, int len)
+unsigned int
+buffer_erase (Buffer *buffer, unsigned int pos, unsigned int len)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
-	assert (len >= 0);
 	assert (buffer->_len >= pos + len);
 	
 	memmove (buffer->_data + pos,
@@ -203,7 +163,6 @@ buffer_erase (Buffer *buffer, int pos, int len)
 		 buffer->_len  - (pos + len));
 	
 	buffer->_len  -= len;
-	//buffer->_data = realloc (buffer->_data, buffer->_len);
 	
 	return buffer->_len;
 }
@@ -212,16 +171,15 @@ buffer_erase (Buffer *buffer, int pos, int len)
 
 
 BufferIterator 
-buffer_find_offset (const Buffer *buffer, int offset, const char* pattern, int len)
+buffer_find_offset (const Buffer *buffer, unsigned int offset, const char* pattern, 
+		    unsigned int len)
 {	
-	int i, j;
+	unsigned int i, j;
 	int match = 1;
 	
 	assert (buffer);
 	assert (buffer->_len >= 0);
-	assert (len >= 0);
 	assert (pattern || len == 0);
-	assert (offset >= 0);
 	assert (offset < buffer->_len);
 	
 	for (i = offset; i < buffer->_len - len; ++i) {
@@ -245,7 +203,6 @@ void
 buffer_clear (Buffer *buffer)
 {
 	assert (buffer);
-	assert (buffer->_len >= 0);
 
 	free (buffer->_data);
 
